@@ -1695,7 +1695,7 @@ CK_DEFINE_FUNCTION(CK_RV, C_SignInit)(
   CK_KEY_TYPE  type = 0;
   CK_ULONG     key_len = 0;
   CK_BYTE      exp[3];
-  CK_BYTE      buf[1024];
+  CK_BYTE      buf[1024] = {0};
   CK_ATTRIBUTE template[] = {
     {CKA_KEY_TYPE, &type, sizeof(type)},
     {CKA_MODULUS_BITS, &key_len, sizeof(key_len)},
@@ -1863,12 +1863,6 @@ CK_DEFINE_FUNCTION(CK_RV, C_Sign)(
     goto sign_out;
   }
 
-  if (op_info.type != YKCS11_SIGN) {
-    DBG("Operation not initialized");
-    rv = CKR_OPERATION_NOT_INITIALIZED;
-    goto sign_out;
-  }
-
   if (session.info.state == CKS_RO_PUBLIC_SESSION ||
       session.info.state == CKS_RW_PUBLIC_SESSION) {
     DBG("User is not logged in");
@@ -1942,7 +1936,7 @@ CK_DEFINE_FUNCTION(CK_RV, C_Sign)(
 
   *pulSignatureLen = sizeof(op_info.buf);
 
-  piv_rv = ykpiv_sign_data(piv_state, op_info.buf, op_info.buf_len, pSignature, pulSignatureLen, op_info.op.sign.algo, op_info.op.sign.key_id);
+  piv_rv = ykpiv_sign_data(piv_state, op_info.buf, op_info.buf_len, op_info.buf, pulSignatureLen, op_info.op.sign.algo, op_info.op.sign.key_id);
   if (piv_rv != YKPIV_OK) {
     if (piv_rv == YKPIV_AUTHENTICATION_ERROR) {
       DBG("Operation requires authentication or touch");
@@ -1958,13 +1952,13 @@ CK_DEFINE_FUNCTION(CK_RV, C_Sign)(
 
   DBG("Got %lu bytes back", *pulSignatureLen);
 #if YKCS11_DBG == 1
-  dump_data(pSignature, *pulSignatureLen, stderr, CK_TRUE, format_arg_hex);
+  dump_data(op_info.buf, *pulSignatureLen, stderr, CK_TRUE, format_arg_hex);
 #endif
 
   if (!is_RSA_mechanism(op_info.mechanism.mechanism)) {
     // ECDSA, we must remove the DER encoding and only return R,S
     // as required by the specs
-    strip_DER_encoding_from_ECSIG(pSignature, pulSignatureLen);
+    strip_DER_encoding_from_ECSIG(op_info.buf, pulSignatureLen);
 
     DBG("After removing DER encoding %lu", *pulSignatureLen);
 #if YKCS11_DBG == 1
@@ -1972,7 +1966,7 @@ CK_DEFINE_FUNCTION(CK_RV, C_Sign)(
 #endif
   }
 
-  op_info.type = YKCS11_NOOP;
+  memcpy(pSignature, op_info.buf, *pulSignatureLen);
 
   rv = CKR_OK;
 
